'use strict';

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i]; return arr2; } else { return Array.from(arr); } }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const resolvedPromise = Promise.resolve();

module.exports = class Emittery {
	constructor() {
		this._events = new Map();
		this._anyEvents = new Set();
	}

	_getListeners(eventName) {
		if (!this._events.has(eventName)) {
			this._events.set(eventName, new Set());
		}

		return this._events.get(eventName);
	}

	on(eventName, listener) {
		this._getListeners(eventName).add(listener);
		return this.off.bind(this, eventName, listener);
	}

	off(eventName, listener) {
		if (!listener) {
			this._getListeners(eventName).clear();
			return;
		}

		this._getListeners(eventName).delete(listener);
	}

	once(eventName) {
		return new Promise(resolve => {
			const off = this.on(eventName, data => {
				off();
				resolve(data);
			});
		});
	}

	emit(eventName, eventData) {
		var _this = this;

		return _asyncToGenerator(function* () {
			yield resolvedPromise;
			const listeners = [].concat(_toConsumableArray(_this._getListeners(eventName))).map((() => {
				var _ref = _asyncToGenerator(function* (listener) {
					return listener(eventData);
				});

				return function (_x) {
					return _ref.apply(this, arguments);
				};
			})());
			const anyListeners = [].concat(_toConsumableArray(_this._anyEvents)).map((() => {
				var _ref2 = _asyncToGenerator(function* (listener) {
					return listener(eventName, eventData);
				});

				return function (_x2) {
					return _ref2.apply(this, arguments);
				};
			})());
			return Promise.all([].concat(_toConsumableArray(listeners), _toConsumableArray(anyListeners)));
		})();
	}

	emitSerial(eventName, eventData) {
		var _this2 = this;

		return _asyncToGenerator(function* () {
			yield resolvedPromise;

			/* eslint-disable no-await-in-loop */
			for (const listener of _this2._getListeners(eventName)) {
				yield listener(eventData);
			}

			for (const listener of _this2._anyEvents) {
				yield listener(eventName, eventData);
			}
			/* eslint-enable no-await-in-loop */
		})();
	}

	onAny(listener) {
		this._anyEvents.add(listener);
		return this.offAny.bind(this, listener);
	}

	offAny(listener) {
		if (!listener) {
			this._anyEvents.clear();
			return;
		}

		this._anyEvents.delete(listener);
	}

	clear() {
		this._events.clear();
		this._anyEvents.clear();
	}

	listenerCount(eventName) {
		if (!eventName) {
			let count = this._anyEvents.size;

			for (const value of this._events.values()) {
				count += value.size;
			}

			return count;
		}

		return this._anyEvents.size + this._getListeners(eventName).size;
	}
};
